var EmailFileController = Class.create({

    config: null,
    contacts: null,
    selectedContactIds: null,

    emailFileDialog: null,
    contactListDialog: null,
    addContactDialog: null,

    _constructor: function(config) {
        this.config = Object.extend(Object.extend({}, EmailFileController.DEFAULT_CONFIG), config);
        this.contacts = [];
        this.selectedContactIds = [];
    },

    setContacts: function(data) {
        this.contacts = [];
        var values = $A(data);

        for (var i = 0, size = values.length; i < size; i = i + 2) {
            var name = values[i];
            var email = i + 1 < size ? values[i + 1] : "";
            this.contacts.push({id: name + email, name: name, email: email});
        }

        this._updateAddressBookState();
    },

    compose: function(filenames) {
        this._hideComposeDialog();
        this._showComposeDialog(filenames);
    },

    _handleComposeFormAddContact: function(dialog) {
        this._hideAddContactDialog();
        this._showAddContactDialog();
    },

    _handleComposeFormGetAddressBook: function(dialog) {
        this._hideContactListDialog();
        this._showContactListDialog();
    },

    _handleComposeFormCancel: function() {
        this._hideComposeDialog();
    },

    _handleComposeFormDone: function(dialog) {
        var message = dialog.getDoneMessage();
        this._hideComposeDialog();
        this._showMessage(message);
    },

    _handleContactListSubmit: function(dialog) {
        var contacts = dialog.getSelectedContacts();
        this._hideContactListDialog();
        this._updateSelectedContacts(contacts);
    },

    _handleContactListCancel: function() {
        this._hideContactListDialog();
    },

    _handleAddContactCancel: function(dialog) {
        this._hideAddContactDialog();
    },

    _handleAddContactDone: function(dialog) {
        this._hideAddContactDialog();
    },

    _showComposeDialog: function(filenames) {
        var viewport = document.viewport.getDimensions();

        var dialog = new EmailFileDialog(this.config.composeFormUrl);
        dialog.setSize(viewport.width * 0.6, Math.min(533, viewport.height * 0.9));
        dialog.show(filenames, {
            onAddContact: this._handleComposeFormAddContact.bind(this),
            onGetAddressBook: this._handleComposeFormGetAddressBook.bind(this),
            onCancel: this._handleComposeFormCancel.bind(this),
            onDone: this._handleComposeFormDone.bind(this)
        });

        this.emailFileDialog = dialog;
    },

    _hideComposeDialog: function() {
        this._hideAddContactDialog();
        this._hideContactListDialog();

        this._disposeDialog(this.emailFileDialog);
        this.emailFileDialog = null;
        this.selectedContactIds = [];
    },

    _showContactListDialog: function() {
        var dialog = new ContactListDialog(this.contacts.clone());
        dialog.setSelectedContacts(this._findSelectedContacts());
        dialog.show({
            onSubmit: this._handleContactListSubmit.bind(this),
            onCancel: this._handleContactListCancel.bind(this)
        });

        this.contactListDialog = dialog;
    },

    _hideContactListDialog: function() {
        this._disposeDialog(this.contactListDialog);
        this.contactListDialog = null;
    },

    _showAddContactDialog: function() {
        var dialog = new FormDialog("AddContactDialog", this.config.addContactUrl);
        dialog.setSize(430, 235);
        dialog.show(Object.extend(new FormDialog.Listener(), {
            onCancel: this._handleAddContactCancel.bind(this),
            onDone: this._handleAddContactDone.bind(this)
        }));

        this.addContactDialog = dialog;
    },

    _hideAddContactDialog: function() {
        this._disposeDialog(this.addContactDialog);
        this.addContactDialog = null;
    },

    _disposeDialog: function(dialog) {
        if (dialog) {
            dialog.dispose();
        }
    },

    _updateSelectedContacts: function(contacts) {
        var ids = this.selectedContactIds.clone();
        var selection = contacts.clone();

        for (var i = 0; i < ids.length; ) {
            var contact = this._findContactById(ids[i]);
            var index = selection.indexOf(contact);

            if (index != -1) {
                selection.splice(index, 1);
                ++i;
            } else {
                this.emailFileDialog.removeRecipient(contact);
                ids.splice(i, 1);
            }
        }

        while (selection.length != 0) {
            var newContact = selection.shift();
            this.emailFileDialog.addRecipient(newContact);
            ids.push(newContact.id);
        }

        this.selectedContactIds = ids;
    },

    _updateAddressBookState: function() {
        if (this.emailFileDialog) {
            this.emailFileDialog.setAddressBookEnabled(this.contacts.length != 0);
        }
    },

    _findSelectedContacts: function() {
        var result = [];
        for (var i = 0; i < this.selectedContactIds.length; ++i) {
            var id = this.selectedContactIds[i];
            for (var j = 0; j < this.contacts.length; ++j) {
                var candidate = this.contacts[j];
                if (candidate.id == id) {
                    result.push(candidate);
                }
            }
        }
        return result;
    },

    _findContactById: function(id) {
        for (var i = 0; i < this.contacts.length; ++i) {
            var candidate = this.contacts[i];
            if (candidate.id == id) {
                return candidate;
            }
        }
        return null;
    },

    _showMessage: function(message) {
        if (message) {
            alert(message);
        }
    }
});

EmailFileController.DEFAULT_CONFIG = {composeFormUrl:"", addContactUrl:""};