var EmailFileDialog = Class.create(ModalDialog, {

    form: null,
    view: null,
    channel: null,
    addressBookEnabled: false,
    done: false,
    doneMessage: null,

    listener: null,

    _constructor: function(url) {
        this.channel = new IFrameChannel({preventCaching: true});
        this.channel.open(url, "post", Object.extend(new Channel.Listener(), {
            onComplete: this._handleFormLoaded.bind(this)
        }));

        this.setTitle(EmailFileDialog.TITLE);
        this.addButton(ModalDialog.BUTTON_OK, EmailFileDialog.Buttons.OK, this._handleSubmitPressed.bind(this));
        this.addButton(ModalDialog.BUTTON_CANCEL, EmailFileDialog.Buttons.CANCEL, this._handleCancelPressed.bind(this));
    },

    getId: function() {
        return "EmailFileDialog";
    },

    getForm: function() {
        return this.form;
    },

    setAddressBookEnabled: function(value) {
        this.addressBookEnabled = value;
        if (this.view) {
            this.view.setAddressBookEnabled(this.addressBookEnabled);
        }
    },

    addRecipient: function(contact) {
        if (this.view) {
            this.view.addRecipient(contact);
        }
    },

    removeRecipient: function(contact) {
        if (this.view) {
            this.view.removeRecipient(contact);
        }
    },

    show: function(filenames, listener) {
        this.form = null;
        this.listener = Object.extend(new EmailFileDialog.Listener(), listener || {});
        this.channel.send({filename: filenames});

        this.setButtonEnabled(ModalDialog.BUTTON_OK, false);
        this.setVisible(true);
    },

    dispose: function() {
        this.form = null;
        this.channel.close();
        this._super.dispose.call(this);
    },

    getDoneMessage: function() {
        return this.doneMessage || "";
    },

    isDone: function() {
        return this.done;
    },

    /** EmailFileView.Listener implementation */
    onAddContact: function() {
        this.listener.onAddContact(this);
    },

    /** EmailFileView.Listener implementation */
    onGetAddressBook: function() {
        this.listener.onGetAddressBook(this);
    },

    _handleSubmitPressed: function() {
        this.setButtonEnabled(ModalDialog.BUTTON_OK, false);

        if (this.form != null) {
            this.channel.submitForm(this.form);
        }

        this.listener.onSubmit(this);
    },

    _handleCancelPressed: function() {
        this.setVisible(false);
        this.channel.close();

        this.listener.onCancel(this);
    },

    _handleFormLoaded: function() {
        var metadata = this.channel.getResponseMetadata();
        this.done = (metadata["x-dialog-control"] || "").toLocaleLowerCase() == "done";
        this.doneMessage = metadata["x-dialog-reason"] || "";

        if (this.isDone()) {
            this.listener.onDone(this);
        } else {
            this._initDialog();
            this.listener.onLoaded(this);
        }
    },

    _initDialog: function() {
        this.getContent().innerHTML = this.channel.getResponseHtml();
        var forms = this.getContent().getElementsByTagName("form");

        if (forms.length != 0) {
            this.form = forms[0];
            Event.observe(this.form, "submit", this._handleSubmitPressed.bind(this));

            this.view = new EmailFileView();
            this.view.setAddressBookEnabled(this.addressBookEnabled);
            this.view.show(this);
        } else {
            this.form = null;

            if (this.view != null) {
                this.view.hide();
            }
            this.view = null;
        }

        this.setButtonEnabled(ModalDialog.BUTTON_OK, true);
    }
});

EmailFileDialog.TITLE = "Email File(s)";
EmailFileDialog.Buttons = {OK: 'Send', CANCEL: 'Cancel'};
EmailFileDialog.Listener = Class.create(ModalDialog.Listener, {
    onAddContact: function(dialog) {},
    onGetAddressBook: function(dialog) {},
    onLoaded: function(dialog) {},
    onDone: function(dialog) {}
});
