var EmailFileView = Class.create({

    recipientField: null,
    contactListButton: null,
    contactListAddButton: null,
    enableMaxDownloadsBox: null,
    maxDownloadsField: null,
    deleteAfterMaxDownloadsBox: null,
    protectWithComboBox: null,
    customPasswordField: null,

    viewStateHandler: null,
    contactListAddHandler: null,
    contactListSelectHandler: null,

    listener: null,

    _constructor: function(doc) {
        function byId(id, d) { return (d || document).getElementById(id); }
        this.recipientField = byId("to", doc);
        this.contactListButton = byId("clbtn", doc);
        this.contactListAddButton = byId("claddbtn", doc);
        this.enableMaxDownloadsBox = byId("emaxdl", doc);
        this.maxDownloadsField = byId("maxdownloads", doc);
        this.deleteAfterMaxDownloadsBox = byId("deleteaftermaxdownloads", doc);
        this.protectWithComboBox = byId("protectwith", doc);
        this.customPasswordField = byId("custompwd", doc);

        new NumberEditor(this.maxDownloadsField).setMinValue(1);
        Utils.setStyle(this.contactListButton, "cursor:pointer");
        Utils.setStyle(this.contactListAddButton, "cursor:pointer");
    },

    addRecipient: function(contact) {
        var contacts = this._parseContacts();
        contacts.push(this._formatContact(contact));
        this._setContacts(contacts);
    },

    removeRecipient: function(contact) {
        var contacts = this._parseContacts();
        var index = contacts.indexOf(this._formatContact(contact));
        if (index != -1) {
            contacts.splice(index, 1);
            this._setContacts(contacts);
        }
    },

    setAddressBookEnabled: function(value) {
        // Utils.setEnabled(this.contactListButton, value);
        Utils.setVisible(this.contactListButton, value);
    },

    show: function(listener) {
        this.listener = Object.extend(new EmailFileView.Listener(), listener || {});
        this._bindEvents();
        this._update();
    },

    hide: function() {
        this._unbindEvents();
        this.listener = null;
    },

    _update: function() {
        var maxDownloadsEnabled = this.enableMaxDownloadsBox.checked;
        Utils.setEnabled(this.maxDownloadsField, maxDownloadsEnabled);
        Utils.setEnabled(this.deleteAfterMaxDownloadsBox, maxDownloadsEnabled);

        if (Object.isElement(this.customPasswordField) && Object.isElement(this.protectWithComboBox)) {
            var customSelected = $F(this.protectWithComboBox) == 'protectwithcustom';
            Utils.setEnabled(this.customPasswordField, customSelected);
        }
    },

    _handleAddContactPressed: function(e) {
        Event.stop(e);
        this.listener.onAddContact(this);
    },

    _handleGetAddressBookPressed: function(e) {
        Event.stop(e);
        this.listener.onGetAddressBook(this);
    },

    _bindEvents: function() {
        this.viewStateHandler = this._update.bind(this);
        this.contactListAddHandler = this._handleAddContactPressed.bindAsEventListener(this);
        this.contactListSelectHandler = this._handleGetAddressBookPressed.bindAsEventListener(this);

        Event.observe(this.contactListButton, "click", this.contactListSelectHandler);
        Event.observe(this.contactListAddButton, "click", this.contactListAddHandler);
        Event.observe(this.enableMaxDownloadsBox, "click", this.viewStateHandler);

        if (Object.isElement(this.protectWithComboBox)) {
            Event.observe(this.protectWithComboBox, "change", this.viewStateHandler);
        }
    },

    _unbindEvents: function() {
        Event.unobserve(this.contactListButton, "click", this.contactListSelectHandler);
        Event.unobserve(this.contactListAddButton, "click", this.contactListAddHandler);
        Event.unobserve(this.enableMaxDownloadsBox, "click", this.viewStateHandler);

        if (Object.isElement(this.protectWithComboBox)) {
            Event.unobserve(this.protectWithComboBox, "change", this.viewStateHandler);
        }

        this.viewStateHandler = null;
        this.contactListAddHandler = null;
        this.contactListSelectHandler = null;
    },

    _formatContact: function(contact) {
        return "\"" + contact.name + "\" <" + contact.email + ">";
    },

    _setContacts: function(contacts) {
        this.recipientField.value = contacts.join(", ");
    },

    _parseContacts: function() {
        var value = new String(this.recipientField.value || "").trim();
        return (value.length != 0) ? value.split(/\s*,\s*/) : [];
    }

});

EmailFileView.Listener = Class.create({
    onAddContact: function(view) {},
    onGetAddressBook: function(view) {}
});