var GenerateKeyDialog = Class.create({

    dialog: null,
    listener: null,

    _constructor: function(id, formDataUrl) {
        this.dialog = new FormDialog(id, formDataUrl);
    },

    isDone: function() {
        return this.dialog.isDone();
    },

    getDoneMessage: function() {
        return this.dialog.getDoneMessage();
    },

    getDownloadUrl: function() {
        return this.dialog.getMetaValue("x-dialog-download-url");
    },

    setSize: function(width, height) {
        this.dialog.setSize(width, height);
    },

    show: function(listener) {
        this.listener = listener || new GenerateKeyDialog.Listener();

        this.dialog.show({
            onLoaded: this._handleLoaded.bind(this),
            onLoadError: this._handleLoadError.bind(this),
            onSubmit: this._handleSubmit.bind(this),
            onCancel: this._handleCancel.bind(this),
            onDone: this._handleDone.bind(this)
        });
    },

    hide: function() {
        this.dialog.hide();
        this.listener = null;
    },

    dispose: function() {
        this.dialog.dispose();
        this.listener = null;
    },

    _handleLoaded: function() {
        this._setSubmitButtonText(GenerateKeyDialog.Buttons.OK);
        this._setCancelButtonText(GenerateKeyDialog.Buttons.CANCEL);
    },

    _handleLoadError: function() {
        this._setSubmitButtonText(FormDialog.Buttons.OK);
    },

    _handleSubmit: function() {
        this._setSubmitButtonText(GenerateKeyDialog.Messages.GENERATING);
    },

    _handleCancel: function() {
        this.dispose();
    },

    _handleDone: function() {
        this.listener.onDone(this);
    },

    _setSubmitButtonText: function(value) {
        this._setButtonText(ModalDialog.BUTTON_OK, value || FormDialog.Buttons.OK);
    },

    _setCancelButtonText: function(value) {
        this._setButtonText(ModalDialog.BUTTON_CANCEL, value || FormDialog.Buttons.CANCEL);
    },

    _setButtonText: function(type, text) {
        this.dialog.getButton(type).value = text;
    }
});

GenerateKeyDialog.Buttons = {OK: 'OK', CANCEL: 'Cancel'};
GenerateKeyDialog.Messages = {GENERATING: "Generating..."};
GenerateKeyDialog.Listener = Class.create({
    onDone: function(dialog) {}
});