var NewDirectoryDialog = Class.create(ModalDialog, {

    dirField: null,
    dirFieldKeyboardHandler: null,
    listener: null,

    _constructor: function() {
        this.dirField = Utils.createTextInput("newDirInputField");

        this.getContent().appendChild(Utils.createLabel(NewDirectoryDialog.Labels.NAME, this.dirField.name));
        this.getContent().appendChild(this.dirField);
        this.addButton(ModalDialog.BUTTON_OK, NewDirectoryDialog.Buttons.OK, this.onSubmit.bind(this));
        this.addButton(ModalDialog.BUTTON_CANCEL, NewDirectoryDialog.Buttons.CANCEL, this.onCancel.bind(this));
        this.setButtonEnabled(ModalDialog.BUTTON_OK, false);
        this.setTitle(NewDirectoryDialog.TITLE);
    },

    getId: function() {
        return "NewDirectoryDialog";
    },

    show: function(listener) {
        this.listener = listener || new NewDirectoryDialog.Listener();
        this._attachHandlers();
        this._updateButtons();
        this.setVisible(true);
    },

    hide: function() {
        this._detachHandlers();
        this.dispose();
    },

    getDirectoryName: function() {
        var result = $F(this.dirField);
        return (result != null) ? result : "";
    },

    setDirectoryName: function(value) {
        this.dirField.value = value || "";
    },

    isCorrect: function() {
        var name = this.getDirectoryName();
        return name != null && name.trim().length > 0;
    },

    onSubmit: function() {
        this.listener.onSubmit(this);
    },

    onCancel: function() {
        this.listener.onCancel(this);
    },

    _attachHandlers: function() {
        this.dirFieldKeyboardHandler = this._updateButtons.bind(this);
        Event.observe(this.dirField, "keyup", this.dirFieldKeyboardHandler);
    },

    _detachHandlers: function() {
        Event.unobserve(this.dirField, "keyup", this.dirFieldKeyboardHandler);
        this.dirFieldKeyboardHandler = null;
    },

    _updateButtons: function() {
        this.setButtonEnabled(ModalDialog.BUTTON_OK, this.isCorrect());
    },

    _requestFocus: function() {
        this._super._requestFocus.call(this, this.dirField);
    }
});

NewDirectoryDialog.TITLE = "New Directory";
NewDirectoryDialog.Labels = {NAME: 'New directory'};
NewDirectoryDialog.Buttons = {OK: 'OK', CANCEL: 'Cancel'};
NewDirectoryDialog.Listener = Class.create({
    onSubmit: function(view) {},
    onCancel: function(view) {}
});