var StorageController = Class.create({

    config: null,
    view: null,
    uploader: null,

    createDirAction: null,
    changeWorkingDirAction: null,
    changeDirUpAction: null,
    downloadFileAction: null,
    deleteFileAction: null,
    renameFileAction: null,
    moveFileAction: null,
    tagFileAction: null,
    refreshAction: null,
    switchUiAction: null,

    _constructor: function(config) {
        this.config = Object.extend(Object.extend({}, StorageController.DEFAULT_CONFIG), config);
        this.view = new StorageView(this.config);
        if (config.uploadAllowed) {
            this.uploader = new Uploader(config);
        }

        var form = $("actionform");
        this.downloadFileAction = new Action("dload", form);
        this.deleteFileAction = new Action("delete", form);
        this.renameFileAction = new Action("rename", form);
        this.moveFileAction = new Action("move", form);
        this.createDirAction = new Action("mkdir", form);
        this.changeWorkingDirAction = new Action("cwd", form);
        this.changeDirUpAction = new Action("cdup", form);
        this.tagFileAction = new Action("ManageTags", form);
        this.cancelUploadAction = new Action("", form);
        this.refreshAction = new Action("refresh", form);
        this.switchUiAction = new Action("uiapplet", form);
    },

    start: function() {
        if (this.uploader) {
            this.uploader.start({
                onStart: this.onStartUpload.bind(this),
                onComplete: this.onCompleteUpload.bind(this),
                onTimeout: this.onTimeoutUpload.bind(this)
            });
        }
        this.view.show(this);
    },

    onStartUpload: function() {
        this._stopInactivityService();
    },

    onCompleteUpload: function() {
        var error = this.uploader.getTransferError();
        if (error) {
            //todo: this.view.showTransferError(error);
        }
        this.refreshAction.perform(this._actionParameters());
    },

    onTimeoutUpload: function() {
        //todo: this.view.showConnectionTimeoutMessage();
        this.refreshAction.perform(this._actionParameters());
    },

    onRenameFile: function(view) {
        var items = view.getSelectedItems();

        if (items.length == 0) {
            view.showEmptySelectionMessage();
        } else if (items.length == 1) {
            var dialog = new RenameFileDialog();
            dialog.show(items.first().path, items.first().node, {
                onSubmit: this._onSubmitRenameFileDialog.bind(this),
                onCancel: function(d) { d.hide(); }
            });
        }
    },

    onMoveFile: function(view, data) {
        this.moveFileAction.perform(Object.extend(this._actionParameters(), {
            filename: data.files,
            destination: data.destination
        }));
    },

    onNewDir: function() {
        var dialog = new NewDirectoryDialog();
        dialog.show({
            onSubmit: this._onSubmitNewDirDialog.bind(this),
            onCancel: function(d) { d.hide(); }
        });
    },

    onChangeDir: function() {
        var dialog = new ChangeDirectoryDialog();
        dialog.show({
            onSubmit: this._onSubmitChangeDirDialog.bind(this),
            onCancel: function(d) { d.hide(); }
        });
    },

    onChangeWorkingDir: function(view, path) {
        if (path == "..") {
            this.changeDirUpAction.perform(this._actionParameters());
        } else {
            this.changeWorkingDirAction.perform(this._actionParameters(path));
        }
    },

    onDownloadFile: function(view, path) {
        this.view.hideMessage();
        this.downloadFileAction.perform(this._actionParameters(path));
    },

    onDeleteFile: function(view) {
        var items = view.getSelectedItems();

        if (items.length == 0) {
            view.showEmptySelectionMessage();
            return;
        }
        if (!view.confirmDeleteMessage()) {
            return;
        }

        var paths = this._itemsAsPaths(items);
        this.deleteFileAction.perform(this._actionParameters(paths));
    },

    onEmailFile: function(view) {
        var items = view.getSelectedItems();

        if (items.length == 0) {
            view.showEmptySelectionMessage();
            return;
        }

        // todo:
        var paths = this._itemsAsPaths(items);
        if (window.adHocController) {
            window.adHocController.compose(paths);
        }
    },

    onTagFile: function(view) {
        var items = view.getSelectedItems();
        if (!this.config.tagFileAllowed || items.length != 1) {
            return;
        }
        if (items.length == 0) {
            view.showEmptySelectionMessage();
            return;
        }

        this.view.hideMessage();
        var dialog = new FormDialog("tag-file-dialog", this.config.manageTagsFormUrl);
        dialog.setSize(470, 180);

        var formData = {filename: this._itemsAsPaths(items).first()};
        dialog.show(Object.extend(new FormDialog.Listener(), {
            onDone: this._onDoneFileTagsDialog.bind(this),
            onCancel: function(d) { d.dispose(); }
        }), formData);
    },

    onViewFile: function(view) {
        var items = view.getSelectedItems();
        if (!this.config.viewFileAllowed || items.length != 1) {
            return;
        }
        if (items.length == 0) {
            view.showEmptySelectionMessage();
            return;
        }

        // todo:
        /*var dialog = new FormDialog("DocumentViewerDialog", this.config.webDocumentViewerUrl);
        var viewport = document.viewport.getDimensions();
        dialog.setSize(viewport.width * 0.8, viewport.height * 0.85);

        var formData = {filename: this._itemsAsPaths(items).first()};
        dialog.show(Object.extend(new FormDialog.Listener(), {
            onCancel: function(d) { d.dispose(); }
        }), formData);*/
        window.docViewer.load(this._itemsAsPaths(items).first());
    },

    onRefreshView: function(view) {
        this.refreshAction.perform(this._actionParameters());
    },

    onSwitchView: function(view) {
        this.switchUiAction.perform();
    },

    _onChangeFileSelection: function() {
        // todo: event bus
        this.view.update();
    },

    _onSubmitNewDirDialog: function(dialog) {
        dialog.hide();
        if (dialog.isCorrect()) {
            this.createDirAction.perform(this._actionParameters(dialog.getDirectoryName()));
        }
    },

    _onSubmitChangeDirDialog: function(dialog) {
        var path = dialog.getPath();
        dialog.hide();
        if (dialog.isCorrect()) {
            this.changeWorkingDirAction.perform(this._actionParameters(path));
        }
    },

    _onSubmitRenameFileDialog: function(dialog) {
        var oldFilename = dialog.getOldFilename();
        var newFilename = dialog.getNewFilename();
        dialog.hide();

        if (oldFilename !== newFilename) {
            this.renameFileAction.perform(Object.extend(this._actionParameters(), {
                oldname: oldFilename,
                newname: newFilename
            }));
        }
    },

    _onDoneFileTagsDialog: function(dialog) {
        dialog.dispose();
        // todo: update tags view
        this.refreshAction.perform(this._actionParameters());
    },

    _itemsAsPaths: function(items) {
        var result = [];
        for (var i = 0; i < items.length; ++i) {
            result.push(items[i].path);
        }
        return result;
    },

    // todo: refactor
    _actionParameters: function(filenames) { return window.createParameters ? window.createParameters(filenames) : {}; },
    _stopInactivityService: function() { window.stopInactivityService && window.stopInactivityService(); }
});

StorageController.DEFAULT_CONFIG = {
    manageTagsFormUrl: "",
    webDocumentViewerUrl: "",
    registerKeystroke: false,
    adHocTransferEnabled: false,
    emailFileAllowed: false,
    viewFileAllowed: false,
    tagFileAllowed: false,
    uploadAllowed: true,
    connectionTimeout: 60000,
    uploadSessionUrl: "",
    uploadProgressUrl: "",
    uploadForms: [],
    uploadFormUrl: ""
};