var AccountView = Class.create({

    expiresOnBox: null,
    expirationField: null,
    assignToGroupBox: null,
    groupsField: null,

    mouseHandler: null,

    _constructor: function(config) {
        this.expiresOnBox = $("expiresondate");
        this.expirationField = $("expirationdate");
        this.assignToGroupBox = $("assigntogroup");
        this.groupsField = $("group");

        this._init(config);
    },

    show: function(listener) {
        this.listener = listener || new AccountView.Listener();
        this._bindEvents();
        this._update();
    },

    hide: function() {
        this.listener = null;
        this._unbindEvents();
    },

    _init: function(config) {
        if (this._groupAssignmentAllowed()) {
            this.assignToGroupBox.disabled = this.groupsField.length == 0;
        }

        var de = new DateEditor(this.expirationField, config.datePattern || "MM/dd/yyyy");
        de.setMinValue(new Date().getTime());
    },

    _update: function() {
        this._updateExpirationDate();
        this._updateGroups();
    },

    _updateExpirationDate: function () {
        if (this.expirationField && this.expiresOnBox) {
            this.expirationField.disabled = !this.expiresOnBox.checked;
        }
    },

    _updateGroups: function() {
        if (this._groupAssignmentAllowed()) {
            this.groupsField.disabled = !this.assignToGroupBox.checked;
        }
    },

    _handleMouseEvent: function() {
        this._update();
    },

    _bindEvents: function() {
        this.mouseHandler = this._handleMouseEvent.bind(this);

        Event.observe(this.expiresOnBox, "click", this.mouseHandler);
        if (this._groupAssignmentAllowed()) {
            Event.observe(this.assignToGroupBox, "click", this.mouseHandler);
        }
    },

    _unbindEvents: function() {
        Event.unobserve(this.expiresOnBox, "click", this.mouseHandler);
        if (this._groupAssignmentAllowed()) {
            Event.unobserve(this.assignToGroupBox, "click", this.mouseHandler);
        }

        this.mouseHandler = null;
    },

    _groupAssignmentAllowed: function() {
        return Object.isElement(this.assignToGroupBox) && Object.isElement(this.groupsField);
    }

});

AccountView.Listener = Class.create({});