var ManageTransfersPane = Class.create(ManageItemsPane, {

    removeButton: null,
    extendButton: null,
    expirationDateField: null,
    confirmRevokeDialog: null,
    confirmExtendDialog: null,
    dateEditor: null,
    dateFormat: null,
    isRevokeConfirmed: false,
    isExtendConfirmed: false,

    _constructor: function(extendButton, revokeButton, selectAllBox, itemSelectionBoxes, expirationDateField) {
        this.extendButton = this.addButton;
        this.revokeButton = this.deleteButton;
        this.expirationDateField = expirationDateField;
        this.dateFormat = "MM/dd/yyyy";
    },

    onRevoke: function(dialog) {
        if (dialog != null) {
            dialog.dispose();
        }
        this.isRevokeConfirmed = true;
        this.revokeButton.click();
    },

    onExtend: function(dialog) {
        if (dialog != null) {
            this.expirationDateField.value = this.dateEditor.getValue();
            dialog.dispose();
        }
        this.isExtendConfirmed = true;
        this.extendButton.click();
    },

    _handleRevoke: function(e) {
        if (!this.isRevokeConfirmed) {
            if (this.getSelectedCount() != 0) {
                this._getRevokeDialog().setVisible(true);
            }
            Event.stop(e);
        }
    },

    _handleExtend: function(e) {
        if (!this.isExtendConfirmed) {
            if (this.getSelectedCount() != 0) {
                this._getExtendDialog().setVisible(true);
            }
            Event.stop(e);
        }
    },

    _getRevokeDialog: function() {
        if (this.confirmRevokeDialog == null) {
            this.confirmRevokeDialog = this._createConfirmDialog("RevokeItemsConfirmDialog", ManageTransfersPane.Messages.CONFIRM_REVOKE, this.onRevoke);
        }
        return this.confirmRevokeDialog;
    },

    _getExtendDialog: function() {
        if (this.confirmExtendDialog == null) {
            var dialog = this._createConfirmDialog("ExtendItemsConfirmDialog", ManageTransfersPane.Messages.CONFIRM_EXTEND, this.onExtend);
            var content = dialog.getContent();
            var input = Utils.createTextInput("expirationdate");
            var label = Utils.createLabel(ManageTransfersPane.Labels.DATE, input.id);
            var format = document.createElement("SPAN");
            format.appendChild(document.createTextNode(this.dateFormat));
            Utils.addCssClass(format, "desc");
            Utils.setStyle(format, "margin-left:6px");

            content.appendChild(label);
            content.appendChild(input);
            content.appendChild(format);

            this.confirmExtendDialog = dialog;
            this.dateEditor = new DateEditor(input, this.dateFormat);
            var currentDate = new Date();
            var defaultDate = new Date();
            defaultDate.setDate(currentDate.getDate() + 7);
            this.dateEditor.setMinValue(currentDate.getTime());
            this.dateEditor.setDefaultValue(defaultDate);
            this.dateEditor.setValue(defaultDate);
        }

        return this.confirmExtendDialog;
    },

    _createConfirmDialog: function(id, message, confirmHandler) {
        var dialog = new ModalDialog(id);
        var messagePanel = document.createElement(Tag.PARAGRAPH);
        Utils.setStyle(messagePanel, "font-weight:bold;");
        messagePanel.appendChild(document.createTextNode(message));
        dialog.getContent().appendChild(messagePanel);

        dialog.addButton(ModalDialog.BUTTON_OK, ManageItemsPane.Buttons.OK, confirmHandler.bind(this, dialog));
        dialog.addButton(ModalDialog.BUTTON_CANCEL, ManageItemsPane.Buttons.CANCEL);

        return dialog;
    },

    /* override */
    _update: function() {
        var count = this.getSelectedCount();
        this.selectAllBox.checked = (this.selectionBoxes.length == count);
        Utils.setEnabled(this.revokeButton, count != 0);
        Utils.setEnabled(this.extendButton, count != 0);
    },

    /* override */
    _bindEvents: function () {
        Event.observe(this.addButton, "click", this._handleExtend.bindAsEventListener(this));
        Event.observe(this.deleteButton, "click", this._handleRevoke.bindAsEventListener(this));
        Event.observe(this.selectAllBox, "click", this._handleSelectAll.bind(this));
        var selectionHandler = this._update.bind(this);

        for (var i = 0; i < this.selectionBoxes.length; ++i) {
            Event.observe(this.selectionBoxes[i], "click", selectionHandler);
        }
    }

});

ManageTransfersPane.Labels = {DATE: "Date:"};
ManageTransfersPane.Buttons = {OK: "OK", CANCEL: "Cancel"};
ManageTransfersPane.Messages = {CONFIRM_REVOKE: "Revoke selected item(s)?", CONFIRM_EXTEND: "Extend selected item(s)?"};
