var DlpSettingsDialog = Class.create({

    dialog: null,

    idBoxes: null,
    selectAllBox: null,
    addButton: null,
    editButton: null,
    deleteButton: null,
    moveUpButton: null,
    moveDownButton: null,
    buttonHandler: null,
    updateHandler: null,
    selectAllHandler: null,
    listOrderField: null,

    listener: null,

    _constructor: function(formUrl) {
        this.dialog = new FormDialog("DlpSettingsDialog", formUrl);
    },

    show: function(listener) {
        this.listener = listener || new DlpSettingsDialog.Listener();
        this.dialog.setSize(650, 390);
        this.dialog.show(Object.extend(new FormDialog.Listener(), {
            onLoaded: this._handleLoaded.bind(this),
            onSubmit: this._handleSubmit.bind(this)
        }));
    },

    hide: function() {
        this.listener = null;
        this._unbindEvents();
        this.dialog.dispose();
    },

    getSelectedCount: function() {
        var count = 0;
        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            if (this.idBoxes[i].checked) { ++count; }
        }
        return count;
    },

    getSelectedIndex: function() {
        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            if (this.idBoxes[i].checked) {
                return i;
            }
        }
        return -1;
    },

    moveItem: function(index1, index2) {
        var tmp = this.idBoxes[index1];
        this.idBoxes[index1] = this.idBoxes[index2];
        this.idBoxes[index2] = tmp;

        this._adjustItemOrder();
        this._moveLocal.bind(this).delay(50, index1, index2);
    },

    _adjustItemOrder: function() {
        var result = "";
        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            result += this.idBoxes[i].value;
            if (i < size - 1) {
                result += "|";
            }
        }
        this.listOrderField.value = result;
    },

    _moveLocal: function(index1, index2) {
        var row1 = Utils.findAncestor(this.idBoxes[index1], "TR");
        var row2 = Utils.findAncestor(this.idBoxes[index2], "TR");
        row1.parentNode.insertBefore(row1, row2);

        this._update();
    },

    _update: function() {
        var selectedCount = this.getSelectedCount();
        var selectedIndex = this.getSelectedIndex();
        var itemCount = this.idBoxes.length;
        var canMove = (selectedCount == 1 && itemCount > 1);

        this.selectAllBox.checked = (itemCount == selectedCount && itemCount != 0);
        Utils.setEnabled(this.selectAllBox, itemCount != 0);
        Utils.setEnabled(this.editButton, selectedCount == 1);
        Utils.setEnabled(this.deleteButton, selectedCount != 0 && itemCount > 0);
        Utils.setEnabled(this.moveUpButton, canMove && selectedIndex != 0);
        Utils.setEnabled(this.moveDownButton, canMove && selectedIndex != itemCount - 1);
    },

    _handleLoaded: function() {
        this.idBoxes = Utils.getFormInputs($("dlpform"), Tag.INPUT_TYPE_CHECKBOX, "dlpid");
        this.selectAllBox = $("selectall");
        this.addButton = $("additem");
        this.editButton = $("edititem");
        this.deleteButton = $("deleteitem");
        this.moveUpButton = $("moveup");
        this.moveDownButton = $("movedown");
        this.listOrderField = $("dlplistorder");

        this._bindEvents();
        this._adjustItemOrder();
        this._update();
    },

    _handleSubmit: function() {
        this.listener.onSubmit(this);
    },

    _handleSelectAll: function() {
        var selected = this.selectAllBox.checked;
        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            this.idBoxes[i].checked = selected;
        }
        this._update();
    },

    _handleButtonPressed: function(e) {
        var button = Event.findAncestor(e, "INPUT");

        if (button == this.addButton) {
            Event.stop(e);
            this.listener.onAdd(this);
        } else if (button == this.editButton) {
            Event.stop(e);
            var id = this.idBoxes[this.getSelectedIndex()].value;
            this.listener.onEdit(id);
        } else if (button == this.deleteButton) {
            this.listener.onDelete(this)
        } else if (button == this.moveUpButton) {
            this.listener.onMoveUp(this);
        } else if (button == this.moveDownButton) {
            this.listener.onMoveDown(this);
        }
    },

    _bindEvents: function() {
        this.buttonHandler = this._handleButtonPressed.bindAsEventListener(this);
        this.updateHandler = this._update.bind(this);
        this.selectAllHandler = this._handleSelectAll.bind(this);

        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            Event.observe(this.idBoxes[i], "click", this.updateHandler);
        }
        Event.observe(this.selectAllBox, "click", this.selectAllHandler);
        Event.observe(this.addButton, "click", this.buttonHandler);
        Event.observe(this.editButton, "click", this.buttonHandler);
        Event.observe(this.deleteButton, "click", this.buttonHandler);
        Event.observe(this.moveUpButton, "click", this.buttonHandler);
        Event.observe(this.moveDownButton, "click", this.buttonHandler);
    },

    _unbindEvents: function() {
        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            Event.unobserve(this.idBoxes[i], "click", this.updateHandler);
        }
        Event.unobserve(this.selectAllBox, "click", this.selectAllHandler);
        Event.unobserve(this.addButton, "click", this.buttonHandler);
        Event.unobserve(this.editButton, "click", this.buttonHandler);
        Event.unobserve(this.deleteButton, "click", this.buttonHandler);
        Event.unobserve(this.moveUpButton, "click", this.buttonHandler);
        Event.unobserve(this.moveDownButton, "click", this.buttonHandler);

        this.buttonHandler = null;
        this.updateHandler = null;
        this.selectAllHandler = null
    }
});

DlpSettingsDialog.Listener = Class.create({
    onAdd: function(d) {},
    onEdit: function(d) {},
    onDelete: function(d) {},
    onMoveUp: function(d) {},
    onMoveDown: function(d) {},
    onSubmit: function(d) {},
    onCancel: function(d) {}
});