var VirtualFileView = Class.create({

    realPathType: null,
    realPathField: null,
    resourcePathType: null,
    resourcesField: null,
    realPathOptionsBoxes: null,
    pgpEncryptionSettingsBtn: null,
    dlpSettingsBtn: null,

    updateViewHandler: null,
    pgpEncryptionSettingsHandler: null,
    dlpSettingsHandler: null,
    listener: null,

    _constructor: function() {
        this.realPathType = $("ptreal");
        this.realPathField = $("realpath");
        this.resourcePathType = $("ptresource");
        this.resourcesField = $("resource");
        this.realPathOptionsBoxes = [$("createifnotexists"),$("searchindex"),$("encryptuploads"),$("dlp")];
        this.pgpEncryptionSettingsBtn = $("encryptuploadssettings");
        this.dlpSettingsBtn = $("dlpsettings");
    },

    show: function(listener) {
        this.listener = listener || new VirtualFileView.Listener();
        this._bindEvents();
        this._update();
    },

    hide: function() {
        this._unbindEvents();
        this.listener = null;
    },

    _update: function() {
        if (this.realPathType.checked) {
            this._setRealPathEnabled(true);
            this._setResourcePathEnabled(false);
        } else if (this.resourcePathType.checked) {
            this._setRealPathEnabled(false);
            this._setResourcePathEnabled(true);
        }
    },

    _setRealPathEnabled: function(enable) {
        Utils.setEnabled(this.realPathField, enable);
        Utils.setEnabled(this.pgpEncryptionSettingsBtn, enable);
        Utils.setEnabled(this.dlpSettingsBtn, enable);

        for (var i = 0; i < this.realPathOptionsBoxes.length; ++i){
            Utils.setEnabled(this.realPathOptionsBoxes[i], enable);
        }
    },

    _setResourcePathEnabled: function(enable) {
        Utils.setEnabled(this.resourcesField, enable);
    },

    _handlePgpEncryptionSettings: function(e) {
        Event.stop(e);
        this.listener.onPgpEncryptionSettings(this);
    },

    _handleDlpSettings: function(e) {
        Event.stop(e);
        this.listener.onDlpSettings(this);
    },

    _bindEvents: function() {
        this.updateViewHandler = this._update.bind(this);
        this.pgpEncryptionSettingsHandler = this._handlePgpEncryptionSettings.bindAsEventListener(this);
        this.dlpSettingsHandler = this._handleDlpSettings.bindAsEventListener(this);

        Event.observe(this.realPathType, "click", this.updateViewHandler);
        Event.observe(this.resourcePathType, "click", this.updateViewHandler);
        Event.observe(this.pgpEncryptionSettingsBtn, "click", this.pgpEncryptionSettingsHandler);
        Event.observe(this.dlpSettingsBtn, "click", this.dlpSettingsHandler);
    },

    _unbindEvents: function() {
        Event.unobserve(this.realPathType, "click", this.updateViewHandler);
        Event.unobserve(this.resourcePathType, "click", this.updateViewHandler);
        Event.unobserve(this.pgpEncryptionSettingsBtn, "click", this.pgpEncryptionSettingsHandler);
        Event.unobserve(this.dlpSettingsBtn, "click", this.dlpSettingsHandler);

        this.updateViewHandler = null;
        this.pgpEncryptionSettingsHandler = null;
        this.dlpSettingsHandler = null;
    }

});

VirtualFileView.Listener = Class.create({
    onPgpEncryptionSettings: function(view) {},
    onDlpSettings: function(view) {},
    onSubmit: function(view) {},
    onCancel: function(view) {}
});