var ProgressMeter = Class.create({

    progressUrl: null,
    channel: null,
    timer: null,

    callback: null,
    listener: null,

    _constructor: function(config) {
        this.progressUrl = config.uploadProgressUrl;

        this.channel = new ScriptChannel({timeout: config.connectionTimeout || 0, preventCaching: true});
        this.timer = new Timer(this._poll.bind(this));
        this.timer.setRepeats(false);
        this.timer.setDelay(1000);
    },

    start: function(listener) {
        this.listener = listener || new ProgressMeter.Listener();
        this.callback = this._handleResponse.bind(this);

        this.channel.open(this.progressUrl, Object.extend(new Channel.Listener(), {
            onTimeout: this._handleTimeout.bind(this)
        }));

        this._poll();
    },

    interrupt: function() {
        this.timer.stop();
        if (this.channel) {
            this.channel.close();
        }
    },

    finish: function() {
        this.timer.stop();
        if (this.channel) {
            this.channel.close();
        }
        this.timer = null;
        this.channel = null;
        this.callback = null;
    },

    _poll: function() {
        this.channel.send("", this.callback);
    },

    _handleResponse: function(data) {
        var copy = Object.extend({}, data);
        var state = (copy.state || "").toUpperCase();

        switch (state) {
            case ProgressMeter.Status.Queued:
            case ProgressMeter.Status.InProgress:
                this._handleProgress(copy);
                break;
            case ProgressMeter.Status.Complete:
                this._handleComplete(copy);
                break;
            case ProgressMeter.Status.Canceled:
                this._handleCancel();
                break;
            case ProgressMeter.Status.Error:
                this._handleError(copy.statusMessage);
                break;
            default:
                this._handleError("Unable to obtain file upload progress");
                break;
        }
    },

    _handleProgress: function(data) {
        this.timer.start();

        var percent = parseInt(data.percent) || -1;
        var statusMessage = data.statusMessage || "";
        var timeMessage = data.timeMessage || "";

        this.listener.onProgress(percent, statusMessage, timeMessage);
    },

    _handleComplete: function(data) {
        this.timer.stop();
        if (data && data.statusMessage && data.timeMessage) {
            this.listener.onProgress(100, data.statusMessage, data.timeMessage);
        }
        this.listener.onComplete();
    },

    _handleCancel: function() {
        this.timer.stop();
        this.listener.onCancel();
    },

    _handleError: function(message) {
        this.timer.stop();
        this.listener.onError(message)
    },

    _handleTimeout: function() {
        this.timer.stop();
        this.listener.onTimeout();
    }
});
ProgressMeter.Listener = Class.create({
    onProgress: function(percent, statusMessage, timeMessage) {},
    onComplete: function() {},
    onCancel: function() {},
    onTimeout: function() {},
    onError: function(message) {}
});
ProgressMeter.Status = {Queued: 'QUEUED', InProgress: 'IN_PROGRESS', Complete: 'COMPLETE', Canceled: 'CANCELED', Error: 'ERROR'};